local wesnoth = wesnoth
local laterlife = laterlife
local ipairs = ipairs
local string = string
local T = wesnoth.require("lua/helper.lua").set_wml_tag_metatable {} 

local width, height, border = wesnoth.get_map_size()

width = (width - 1) * 0.5

local mutation_probability = 0.20

local function select_random_array_index(array)
    return array[wesnoth.random(#array)]
end

local mutation_table = {
    { {6,4}, {"Wog", "Sm", "Wwrg"}, },
    { {2,4}, {"Wwrg", "Wwf", "Wog", "Dd", "Gxua"}, },
    { {4,4}, {"Wwf", "Dd", "Sm", }, },    
    { {1,5}, {"Wo", "Wwrg", "Wog", "Sm"}, },     
    { {3,5}, {"Wo", "Wwrg", "Wwf", "Sm", "Ss", "Gg^Fp"}, },
    { {4,5}, {"Wwf", "Dd", "Sm", "Ss", "Gg^Fp", "Hh", "Hh^Fp",}, },
    { {5,5}, {"Wo", "Ss", "Sm",}, },                 
    { {6,5}, {"Gg", "Wo", "Wwf", "Wog"}, }, 
    { {8,5}, {"Dd", "Mm", "Wwf", "Wo", "Hh", "Hh^Fp", "Gg^Fp"}, },
    { {2,6}, {"Wwrg", "Ss", "Gll", "Uu"}, },
    { {3,6}, {"Gll^Fp", "Dd", "Hhd", "Hhd^Uf","Gll"}, },
    { {4,6}, {"Gd^Fp", "Wwf", "Dd", "Sm", "Gd^Uf"}, },    
    { {5,6}, {"Wo", "Wwf"}, },   
    { {6,6}, {"Wo", "Dd", "Sm"}, },           
    { {7,6}, {"Wo", "Dd", "Sm", "Hh", "Hh^Fp", "Gg"}, },               
    { {8,6}, {"Ss", "Wwf", "Wwrg", "Gxua", "Hh", "Hhd", "Hh^Fp", "Dd", "Mm", "Md"}, },    
    { {1,7}, {"Gll^Fp", "Hhd", "Ss", "Wwf", "Gll", "Wwrg"}, },
    { {5,7}, {"Mm", "Md", "Hh", "Hhd", "Gg", "Gg^Fp", "Hh^Fp", "Ss", "Sm", "Wwf", "Wo", "Dd", "Chs", }, },      
    { {3,7}, {"Gll^Fp", "Hhd", "Uu", "Uu^Uf", "Hhd^Fp", "Wwf", "Gll"}, },    
    { {6,7}, {"Sm", "Wog", "Wo", "Dd", "Gll", "Hhd", "Hhd^Uf"}, },
    { {8,7}, {"Wog", "Wo", "Gll", "Gll^Fp", "Chs", "Dd"}, },    
    { {1,8}, {"Gd", "Hhd", "Hh", "Hh^Fp", "Wwf", "Ss", "Mm", "Dd"}, },
    { {2,8}, {"Gd", "Hhd", "Hh", "Hh^Fp", "Wwf", "Ss", "Mm"}, },        
    { {3,8}, {"Hh", "Hhd", "Hh^Fp", "Hhd^Fp", "Hh^Uf", "Gg^Uf", "Dd","Ss"}, },    
    { {4,8}, {"Gd", "Hhd", "Dd"}, },    
    { {5,8}, {"Hh", "Hhd^Fp", "Gll^Fp", "Dd^Do", "Gll", "Gg^Ve", "Ss"}, },
    { {6,8}, {"Wo", "Sm", "Ss", "Dd", "Rb", "Wwrg"}, },    
    { {7,8}, {"Wo", "Sm", "Ss", "Dd", "Rb", "Wwrg"}, },    
    { {8,8}, {"Hh", "Gll^Fp", "Ss", "Hh^Uf", "Ss^Fp"}, },    
    { {1,9}, {"Gg", "Hh", "HH^Fp", "Dd", "Wo"}, }, 
    { {2,9}, {"Dd", "Gg", "Gd^Uf", "Gg^Fp", "Hh", "Hh^Fp"}, },      
    { {4,9}, {"Wo", "Ss", "Sm^Uf", "Rb"}, },            
    { {5,9}, {"Wo", "Ss", "Sm^Uf", "Rb", "Wog", "Wwrg"}, },                
    { {6,9}, {"Hh^Fp", "Gg", "Gg^Uf", "Hh^Uf", "Dd^Do"}, },
    { {8,9}, {"Hh", "Gll^Fp", "Hh^Fp", "Hh^Uf", "Ss^Fp"}, },        
    { {1,10}, {"Hh", "Gd^Fp", "Ss", "Hh^Uf"}, },
    { {2,10}, {"Gg", "Ss", "Hh^Fp", "Uu", "Ss^Fp", "Wo"}, },    
    { {3,10}, {"Ss", "Ss^Fp", "Gd^Fp", "Hh^Fp", "Gd^Uf"}, },
    { {4,10}, {"Sm", "Ss", "Wwf"}, },
    { {5,10}, {"Gd", "Rb", "Hh", "Hh^Fp"}, },       
    { {6,10}, {"Gd", "Rb", "Wwf", "Sm", "Ss", "Hh", "Hh^Fp", "Mm", "Md"}, },    
    { {8,10}, {"Gd", "Hh", "Hh^Fp", "Gg^Uf", "Ss", "Gg^Ve", "Mm", "Chs"}, },           
    { {1,11}, {"Gg", "Mm", "Hh", "Gg^Fp", "Hh^Fp", "Hh^Uf"}, },    
    { {3,11}, {"Mm", "Hh^Fp", "Ss^Fp", "Ss", "Gg^Fp", "Dd", "Sm"}, },    
    { {5,11}, {"Sm", "Wo"}, },
    { {6,11}, {"Gg", "Hh", "Hh^Fp", "Ss", "Gg^Ve"}, },    
    { {3,12}, {"Dd", "Mm", "Gg"}, },
    { {6,12}, {"Gg", "Gd", "Hh", "Hhd", "Hh^Fp", "Ss", }, },    
    { {7,12}, {"Gg", "Gd", "Hhd", "Hh^Fp", "Ss", }, },        
    { {7,13}, {"Dd", "Gg", "Gg^Ve"}, },    
}

local function get_mirror (x) 
    return width + 2 - (x - width) 
end
    
local function apply_mutation (table_row)
    local x = table_row[1][1]
    local x2 = get_mirror(x)
    local y = table_row[1][2]
    local types_array = table_row[2]
    local selected = select_random_array_index(types_array)
    wesnoth.set_terrain(x,y, selected)
    wesnoth.set_terrain(x2,y,selected)
end

for_each_index(mutation_table, function (index, element) 
    if wesnoth.random() < mutation_probability then
        apply_mutation(element)
    end
end)
