local wesnoth = wesnoth
local laterlife = laterlife
local ipairs = ipairs
local string = string
local T = wesnoth.require("lua/helper.lua").set_wml_tag_metatable {} 

local width, height, border = wesnoth.get_map_size()

width = (width - 1) * 0.5

local mutation_probability = 0.10

local function select_random_array_index(array)
    return array[wesnoth.random(#array)]
end

local mutation_table = {
    { {3,4}, {"Gd"}, },    
    { {4,4}, {"Wwg", "Wog", "Ss", "Sm", "Dd", "Gd^Fp"}, },
    { {5,4}, {"Wwg", "Wog", "Ss", "Sm", "Dd", "Gd^Fp", "Hhd",}, },
    { {6,4}, {"Wwf", "Wog", "Wwrg", "Gd", "Dd","Rb^Uf", "Rb",}, },    
    { {1,5}, {"Wwg", "Wog", "Wwrg", "Ss", "Sm", "Hhd", "Hhd^Fp", "Gd", "Gd^Fp", "Uu", "Uu^Uf", "Uu^Dr",}, },
    { {2,5}, {"Rb", "Sm", "Wwrg", "Wwf",}, },
    { {3,5}, {"Wwg", "Sm", "Wwf", "Dd",}, },
    { {4,5}, {"Gd", "Wog", "Wwf", "Wwrg",}, },
    { {5,5}, {"Hhd", "Sm", "Wwf", "Wwg",}, },
    { {6,5}, {"Hhd", "Sm", "Gd", "Hhd^Fp",}, },    
    { {1,6}, {"Uu^Vud", "Uu^Fi", "Gd", "Gd^Fp", "Ss", "Sm", "Hh", "Hhd", "Wwf", "Wog", "Wwrg",}, },
    { {2,6}, {"Sm", "Wwg", "Rb", "Rb^Uf",}, },
    { {3,6}, {"Wwf", "Wwg", "Sm", "Hhd", "Hhd^Fp",}, },
    { {4,6}, {"Md", "Mm", "Sm", "Hhd","Hhd^Fp", "Wwf", "Wwg", "Wog", "Wwrg","Uu", "Uu^Fi", "Rd^Uf", "Rd","Gd^Fp", }, },
    { {5,6}, {"Sm", "Gd^Uf", "Wwf", "Gd^Fp",}, },
    { {6,6}, {"Md", "Mm", "Wwf", "Sm","Hhd", "Hhd^Fp", "Wwf", "Uu", "Uu^Fi", "Gd^Fp", }, },
    { {7,6}, {"Wwf", "Dd", "Sm", "Hhd",}, },
    { {8,6}, {"Sm", "Rb", "Wwrg", "Dd",}, },
    { {9,6}, {"Sm", "Dd", "Rb", "Md", "Uu", "Uu^Uf", "Rb^Uf",}, },    
    { {1,7}, {"Ss", "Uu", "Wwg", "Wog", "Dd", "Uu^Uf", "Rb^Uf",}, },
    { {2,7}, {"Gg", "Gd", "Hhd", "Hh", "Hh^Fp", "Hhd^Fp", "Sm", "Ss", "Ss^Fp","Uh","Wwg","Wwrg","Wwf",}, },
    { {4,7}, {"Rb", "Rb^Uf", "Sm", "Wwg", "Dd",}, },
    { {5,7}, {"Mm", "Md", "Hh", "Hhd", "Hh^Fp", "Hhd^Fp", "Gd", "Gd^Fp", "Gd^Uf", "Wog", "Wwg", "Qxe",}, },    
    { {6,7}, {"Rb", "Rb^Uf", "Sm", "Wwg", "Dd",}, },
    { {7,7}, {"Rb", "Rb^Uf", "Sm", "Wwf", "Wwrg",}, },
    { {8,7}, {"Gd", "Gd^Fp", "Ss", "Ss^Fp", "Hhd",}, },    
    { {9,7}, {"Mm", "Md", "Gd^Fp", "Hhd^Fp", "Hhd", "Uu", "Uu^Uf", "Uh", "Wog", "Wwrg",}, },    
    { {2,8}, {"Gg", "Gg^Fp", "Hhd", "Hh^Fp", "Dd", "Hh^Uf", "Rb^Rd", "Rb",}, },    
    { {3,8}, {"Gg", "Gg^Fp", "Hhd^Fp",}, },        
    { {4,8}, {"Sm", "Wwg", "Dd", "Dd^Uf"}, },
    { {5,8}, {"Mm", "Md", "Dd", "Dd^Uf", "Wwf", "Wwrg", "Wog", "Wwg", "Hhd^Fp", "Hh", "Hh^Fp",}, },
    { {7,8}, {"Hhd", "Mm", "Gd^Fp", "Uu^Uf"}, }, 
    { {9,8}, {"Hhd", "Hh^Fp", }, },        
    { {1,9}, {"Wwg", "Sm", "Ss", }, },  
    { {4,9}, {"Ss", "Hh^Fp", "Hh", "Dd", }, },      
    { {5,9}, {"Wwrg", "Sm", "Rb", "Wog", "Wwg", }, }, 
    { {7,9}, {"Wwrg", "Sm", "Rb", "Rb^Uf", "Wwg", }, },        
    { {8,9}, {"Wwrg", "Sm", "Rb", "Rb^Uf", "Wwg", "Dd", }, },
    { {9,9}, {"Dd", "Gd", "Hhd", "Hhd^Uf", "Hhd^Fp", "Gd^Fp", }, },
    { {1,10}, {"Mm", "Md", "Hhd", "Gd^Fp", "Gg^Fp", "Gg", "Uh", "Hhd^Vhh", }, },            
    { {2,10}, {"Wwrg", "Sm", "Rb", "Rb^Uf", "Wwg", }, },        
    { {3,10}, {"Ss", "Hh^Fp", "Gd^Fp", "Dd", }, },     
    { {4,10}, {"Hh", "Hh^Uf", "Gd^Fp", "Ss^Fp", }, },   
    { {5,10}, {"Hh", "Hh^Uf", "Hh^Fp", "Ss", "Wwrg", }, },  
    { {6,10}, {"Ss", "Hh^Fp", "Gd^Fp", "Dd", }, }, 
    { {7,10}, {"Ss", "Hh^Fp", "Hh", "Dd", }, },     
    { {8,10}, {"Wwrg", "Sm", "Rb", "Rb^Uf", "Wwg", }, },
    { {9,10}, {"Hhd", "Md", "Uh", }, },         
    { {2,11}, {"Hhd", "Hh^Fp", "Gg", }, },  
    { {3,11}, {"Hhd", "Hh^Fp", "Gg", "Ss", "Sm", }, },
    { {4,11}, {"Gg", "Rb^Uf", "Sm", "Sm^Uf", }, },           
    { {5,11}, {"Gg", "Sm", }, }, 
    { {7,11}, {"Sm", "Wwg", "Wwrg", "Rb", }, },           
    { {8,11}, {"Hhd^Fp", "Uh", "Hhd", "Dd^Dr", }, },             
    { {9,11}, {"Hhd^Fp", "Uu", "Sm", "Ss", }, },
    { {2,12}, {"Hhd^Fp", "Dd", "Gg", "Hhd", }, },
    { {3,12}, {"Hhd^Fp", "Hhd", "Mm", "Ss", "Sm", }, },          
    { {4,12}, {"Ss", "Sm^Uf", "Hhd", "Gg^Fp", }, },      
    { {5,12}, {"Hhd^Fp", "Hhd", "Mm", "Ss", "Sm", }, },  
    { {6,12}, {"Ss", "Ss^Fp", "Hhd", "Gg^Fp", }, },
    { {7,12}, {"Sm", "Wwrg", "Dd", }, },
    { {9,12}, {"Mm", "Ss^Vhs", "Hhd", "Chs", }, },                  
    { {2,13}, {"Wwg", "Sm", }, },         
    { {3,13}, {"Gg", "Gg^Fp", "Hhd^Fp", }, },
    { {6,13}, {"Hh", "Gg^Fp", "Hhd^Fp", }, },
    { {7,13}, {"Sm", "Dd", "Dd^Dr", }, },                     
    { {7,14}, {"Dd", "Wwf", "Sm", }, },           
     
}

local function get_mirror (x) 
    return width + 2 - (x - width) 
end
    
local function apply_mutation (table_row)
    local x = table_row[1][1]
    local x2 = get_mirror(x)
    local y = table_row[1][2]
    local types_array = table_row[2]
    local selected = select_random_array_index(types_array)
    wesnoth.set_terrain(x,y, selected)
    wesnoth.set_terrain(x2,y,selected)
end

for_each_index(mutation_table, function (index, element) 
    if wesnoth.random() < mutation_probability then
        apply_mutation(element)
    end
end)
