local helper = wesnoth.require("helper")
local utils = wesnoth.require("wml-utils")
local random_factions_enabled = wml.variables["random_factions_checkbox"] or false

-- generic functions

function ironic_helper_shuffle (t) 
    local danger_area = map(t, function (i,v) return v end)
    helper.shuffle(danger_area)
    return danger_area
end

local factions_and_recruits = {}
factions_and_recruits[1] = { "Cavalryman","Horseman","Spearman","Fencer","Heavy Infantryman","Bowman","Mage","Merman Fighter", }
factions_and_recruits[2] = { "Skeleton","Skeleton Archer", "Ghost","Dark Adept","Ghoul", }
factions_and_recruits[3] = { "Elvish Fighter","Elvish Archer","Mage","Elvish Shaman","Elvish Scout","Wose","Merman Hunter", }
factions_and_recruits[4] = { "Drake Burner","Drake Clasher","Drake Glider","Drake Fighter","Saurian Skirmisher","Saurian Augur", }
factions_and_recruits[5] = { "Dwarvish Guardsman","Dwarvish Fighter","Dwarvish Ulfserker","Dwarvish Thunderer","Thief","Poacher","Footpad","Gryphon Rider", }
factions_and_recruits[6] = { "Orcish Grunt","Troll Whelp","Wolf Rider","Orcish Archer","Orcish Assassin","Naga Fighter", }
level_zero_recruits      = { "Peasant", "Woodsman", "Goblin Spearman", "Walking Corpse", "Vampire Bat", }
bonus_recruits           = { "Giant Spider", "Elvish Lord", "Orcish Leader", "Mermaid Initiate", }

local bonus_recruit_chance = 0.05
local bonus_side_one = wesnoth.random() < bonus_recruit_chance
local bonus_side_two = wesnoth.random() < bonus_recruit_chance

local function choose_n_from_m(n,m)
    local ticket_bowl = initialize_array(m, identity)
    ticket_bowl = ironic_helper_shuffle(ticket_bowl)
    ticket_bowl = array_filter(ticket_bowl, function(i,v) return i <= n end)
    return ticket_bowl
end 

local function choose_n_units_from_array (t, n) 
    local response = {}
    local shuffler = initialize_array(#t, identity)
    shuffler = ironic_helper_shuffle(shuffler)
    for_each_index(shuffler, function (index,value) if index <= n then table.insert(response, t[value]) end end)
    return response
end 

local function replace_side_recruits(side_number, array)
    local recruits = map(array, function (i,v) return v end)
    wesnoth.wml_actions.modify_side({side=side_number, recruit=table.concat(recruits, ",")})
end

local function choose_n_random_from_array(array, n)
    local ticket_bowl = initialize_array(#array, identity)
    ticket_bowl = ironic_helper_shuffle(ticket_bowl)
    choices = initialize_array(#n, identity)
    choices = map(choices, function (i,v) return array[ticket_bowl[i]] end)
    return choices 
end

local side_one_tickets = choose_n_from_m(3,#factions_and_recruits)
local side_two_tickets = choose_n_from_m(3,#factions_and_recruits)

local side_one_zero_ticket  = wesnoth.random(#level_zero_recruits)
local side_two_zero_ticket  = (side_one_zero_ticket - 1 + wesnoth.random(#level_zero_recruits - 1)) % #level_zero_recruits + 1

local side_one_recruits = {}
local side_two_recruits = {}

side_one_recruits = array_merge(side_one_recruits, choose_n_units_from_array(factions_and_recruits[side_one_tickets[1]], 4))
side_one_recruits = array_merge(side_one_recruits, choose_n_units_from_array(factions_and_recruits[side_one_tickets[2]], 2))
side_one_recruits = array_merge(side_one_recruits, choose_n_units_from_array(factions_and_recruits[side_one_tickets[3]], 2))

side_two_recruits = array_merge(side_two_recruits, choose_n_units_from_array(factions_and_recruits[side_two_tickets[1]], 4))
side_two_recruits = array_merge(side_two_recruits, choose_n_units_from_array(factions_and_recruits[side_two_tickets[2]], 2))
side_two_recruits = array_merge(side_two_recruits, choose_n_units_from_array(factions_and_recruits[side_two_tickets[3]], 2))

table.insert(side_one_recruits, level_zero_recruits[side_one_zero_ticket])
table.insert(side_two_recruits, level_zero_recruits[side_two_zero_ticket])



if random_factions_enabled == true then 

    if bonus_side_one == true then
        side_one_recruits[wesnoth.random(#side_one_recruits)] = (choose_n_units_from_array(bonus_recruits, 1))[1]
    end
    
    if bonus_side_two == true then
        side_two_recruits[wesnoth.random(#side_two_recruits)] = (choose_n_units_from_array(bonus_recruits, 1))[1]
    end    


    replace_side_recruits(1, side_one_recruits)
    replace_side_recruits(3, side_two_recruits)
        
end








